import {ActionContext} from "vuex";
import * as seed from "@/seed";
import BilliDB from "@/indexdDB";
import { v4 as uuidv4 } from 'uuid';
import {SEED_VERSION} from "@/seed";


const db = new BilliDB("billibox", 3);


const state = {
    isInitialized: false,
    cameras: [],
    brands: [],
    conditions: [],
    buildtypes: []

};

const getters = {
    cameras: (state: any) => {
       return state.cameras.sort();
    },
    camera: (state: any) => (id: any) => state.cameras.find((camera: any) => camera.id === id),
    activeCamera: (state: any) => state.cameras.find((camera:any) => camera.active ),

    brands: (state: any) => state.brands,
    brand: (state: any) => (id: any) => state.brands.find((brand: any) => brand.schluessel === id),

    conditions: (state: any) => state.conditions,
    condition: (state: any) => (id: any) => state.conditions.find((condition: any) => condition.schluessel === id),

    buildtypes: (state: any) => state.buildtypes,
    buildtype: (state: any) => (id: any) => state.buildtypes.find((buildtype: any) => buildtype.schluessel === id),

    isInitialized: (state:any ) => state.isInitialized,

};
const mutations = {
    setCameras(state:any, payload:any) {
        console.log("mutation.setCameras");
        const camerasDA = [];
        payload.map((camera:any) => {
            state.cameras.push({
                ...camera,
                edit: false,
                active: false
            })
        })
        console.log("STORE", state.cameras);
    },
    storeCamera(sate:any, payload:any) {
       // console.log("mutation.storeCamera", payload)
        const objCamera = state.cameras.find((camera:any) => camera.id === payload.id);
        if(objCamera) {
   //         console.log("found existing Camera", objCamera.id);
            payload.edit =false;
            objCamera.name = payload.name;
        } else {
     //       console.log("creating new Camera");

            payload.edit = false;
            state.cameras.push(payload);

        }
        return payload.id;
    },
    setCameraActiveState(state:any, payload:any) {
     //   console.log("setCameraActiveState", payload);
        state.cameras.map((camera:any) => camera.active= false);
        const objCamera = state.cameras.find((camera:any) => camera.id === payload.id);
        objCamera.active = payload.active;
    },
    setCameraEditState(state:any, payload:any) {
        state.cameras.map((camera:any) => camera.edit= false);
        const objCamera = state.cameras.find((camera:any) => camera.id === payload.id);
        objCamera.edit = payload.edit;
    },
    setBrands(state:any, payload:any) {
       // console.log("mutation.setBrands");
        state.brands = payload;
       // console.log("STORE", state.brands);
    },
    setConditions(state:any, payload:any) {
        //console.log("mutation.setConditions");
        state.conditions = payload;
        //console.log("STORE", state.conditions);
    },
    setBuildtypes(state:any, payload:any) {
        //console.log("mutation.setBuildtypes");
        state.buildtypes = payload;
        //console.log("STORE", state.buildtypes);
    },
    setInitialized(state:any, payload:boolean) {
        state.isInitialized = payload;
    }
};
const actions = {
    fetchCameras(context:ActionContext<any, any>) {
        console.log("action.fetchCameras")
        db.getItems("cameras").then((cameras) => {
            console.log("From DB", cameras);
            context.commit("setCameras", cameras);

        })
    },
    fetchBrands(context:ActionContext<any, any>) {
        context.commit("setBrands", seed.brands);
    },
    fetchBuildTypes(context:ActionContext<any, any>) {
        context.commit("setBuildtypes", seed.buildtype);
    },
    fetchCondition(context:ActionContext<any, any>) {
        context.commit("setConditions", seed.conditions);
    },
    setCameraActiveState(context:ActionContext<any, any>, payload:any) {
        console.log("action.setCameraActiveState", payload)
        const cameraObj = state.cameras.find( (camera:any) => camera.id === payload.cameraId);
        context.commit("setCameraActiveState", {id: payload.cameraId, active:payload.active});

    },

    setCameraEditState(context:ActionContext<any, any>, payload:any) {
        console.log("action.setCameraEditState", payload)
        const cameraObj = state.cameras.find( (camera:any) => camera.id === payload.cameraId);
        context.commit("setCameraEditState", {id: payload.cameraId, edit:payload.edit});

    },

    storeCamera(context:ActionContext<any, any>, payload:any) {
       // console.log("action.storeCamera", payload)

        if(!payload.id) payload.id = uuidv4();
        //console.log(" ... with id ", payload.id);

        //console.log("Try to save to indexDB");
        db.saveItem(payload , "cameras").then( (res) => {
            console.log("DB-Save resolved", res )}
        )


        context.commit("storeCamera", payload);
        context.commit("setCameraActiveState", {cameraId:payload.id, active:true})

    },
    initialize(context:ActionContext<any, any>, payload) {

        db.getDb();
        const promises = [];
        const seedVersion = localStorage.getItem("SEED_VERSION") || 0;
        if(payload.seed && seedVersion < SEED_VERSION ) {

            console.log("SEEDING database");

            promises.push(db.saveItems(seed.brands, "brands"));
            promises.push(db.saveItems(seed.conditions, "conditions"));
            promises.push(db.saveItems(seed.buildtype, "buildtypes"));
            promises.push(db.saveItems(seed.cameras, "cameras"));

            localStorage.setItem("SEED_VERSION", seed.SEED_VERSION.toString())
        } else {
            console.log("SEEDING skipped");
        }
        Promise.all(promises).then( (e) => {
                console.log("All written to store")
            }
        )

        context.dispatch("fetchCameras");
        // context.dispatch("fetchBrands");
        // context.dispatch("fetchBuildTypes");
        // context.dispatch("fetchCondition");
        // context.commit("setInitialized", true);

    }
};
const modules = {};

const cameraModule = {
    state,
    getters,
    mutations,
    actions,
    modules
}

export default cameraModule